/* $Id: lcdprint.c,v 1.3 2005/05/01 10:29:04 dijkstra Exp $ */

/*
 * Copyright (c) 2005 Willem Dijkstra
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdlib.h>
#include <limits.h>
#include <sysexits.h>
#include <string.h>

#include "driver.h"
#include "error.h"
#include "lcd.h"
#include "sylcd.h"
#include "xmalloc.h"

/* Decode <special char number>:<row 1 bits>:<row 2 bits>:...:<row 8 bits>
 * bits are written as 1 or 0, bitmasks are considered to be lsb
 */
char
decode_specialchar(char *customdef, struct specialchar *data)
{
    char *word, *rest;
    int row = 0;
    int i;
    char ch;

    rest = customdef;
    word = strsep(&rest, ":");

    if (word == NULL || rest == NULL) {
	fatal("expected special string of the form <char number>:<bitrow 1>:<bitrow 2>:...");
    }

    ch = strtol(word, NULL, 0);

    while (rest != NULL && row < 8) {
	word = strsep(&rest, ":");
	data->line[row] = 0;
	for (i = 0; i < strlen(word); i++) {
	    data->line[row] <<= 4;
	    data->line[row] |= ((word[i] == '0') ? 0 : 1);
	}
	row++;
    }

    return ch;
}
int
main(int argc, char *argv[])
{
    struct specialchar customchar;
    char *customdef;
    char *lcdconnect;
    char ch;
    char specialch;
    int i;
    int reset;
    int special;
    int x, y;

    customdef = NULL;
    flag_debug = 0;
    lcdconnect = DEFAULT_LCD_CONNECTION;
    reset = -1;
    special = -1;
    x = -1;
    y = -1;

    while ((ch = getopt(argc, argv, "c:dl:rs:vx:y:")) != -1) {
	switch (ch) {
	case 'c':
	    customdef = xstrdup(optarg);
	    break;

	case 'd':
	    flag_debug += 1;
	    break;

	case 'l':
	    if (optarg) {
		lcdconnect = xstrdup(optarg);
	    } else {
		fatal("Need a <lcd connect>");
	    }
	    break;

	case 'r':
	    reset = 1;
	    break;

	case 's':
	    special = 1;
	    specialch = strtol(optarg, NULL, 0);
	    break;

	case 'x':
	    x = strtol(optarg, NULL, 0);
	    break;

	case 'y':
	    y = strtol(optarg, NULL, 0);
	    break;

	case 'v':
	    info("lcdprint version " SYLCD_VERSION " (" LCD_NAME " / %dx%d)", LCD_X, LCD_Y);
	    /* drop through */

	default:
	    info("usage: %s [-c num,bitrow1,..] [-d] [-l <lcd connection>]", __progname);
	    info("\t[-r] [-s num] [-t <text>] [-x <x> -y <y>] [<text>]\n");
	    info("\t-c num:bitrow1:...  define special character num");
	    info("\t                    num = 0x%02x - 0x%02x, %d bitrows of %d bits",
		 LCD_SPECIAL, LCD_SPECIAL_END, LCD_SPECIAL_HEIGTH, LCD_SPECIAL_WIDTH);
	    info("\t-d                  enable debug information");
	    info("\t-l <lcd connection> driver dependent lcd connection");
	    info("\t                    default: " DEFAULT_LCD_CONNECTION);
	    info("\t-r                  reset lcd");
	    info("\t-s num              print special character num");
	    info("\t-v                  version information");
	    info("\t-x <x> -y <y>       move lcd cursor to x, y");
	    exit(EX_USAGE);
	}
    }

    lcd_connect(lcdconnect, 0);

    if (customdef != NULL) {
	ch = decode_specialchar(customdef, &customchar);
	driver_set_specialchar(ch, &customchar);
    }

    if (reset != -1) {
	lcd_flush();
	driver_reset();
    }

    if (x != -1 && y != -1) {
	driver_set_xy(x, y);
    } else if (x != -1 || y != -1) {
	fatal("set both x and y values when positioning");
    }

    if (special != -1) {
	driver_nprintf(2, &specialch);
    }

    argc -= optind;
    argv += optind;

    for (i = 0; i < argc; i++) {
	driver_printf(argv[i]);
    }

    return 0;
}
